/*
 * sysdep.h -- Version dependencies, to compile with both 1.2.x and 2.0.x
 */

/* 1.3.70 changed the semantics of irq request/free to allow shared irq's */

#if LINUX_VERSION_CODE > 0x010345 /* 1.3.70 and newer */

#  define REQUEST_IRQ(irq,fun,fla,nam) request_irq(irq,fun,fla,nam,NULL)
#  define FREE_IRQ(irq) free_irq(irq,NULL)

#else

#  define REQUEST_IRQ(irq,fun,fla,nam) request_irq(irq,fun,fla,nam)
#  define FREE_IRQ(irq) free_irq(irq)

#endif


#if LINUX_VERSION_CODE < 0x010300 /* 1.3-any and later */
/*******************************************************************
 * 1.3.0 added the put_user size-independent macro. Copy it here
 *
 * This code comes from the kernel (v 2.0.x)
 */


#define put_user(x,ptr) __put_user((unsigned long)(x),(ptr),sizeof(*(ptr)))
#define get_user(ptr) ((__typeof__(*(ptr)))__get_user((ptr),sizeof(*(ptr))))

/*
 * This is a silly but good way to make sure that
 * the __put_user function is indeed always optimized,
 * and that we use the correct sizes..
 */
extern int bad_user_access_length(void);

/*
 * dummy pointer type structure.. gcc won't try to do something strange
 * this way..
 */
struct __segment_dummy { unsigned long a[100]; };
#define __sd(x) ((struct __segment_dummy *) (x))
#define __const_sd(x) ((const struct __segment_dummy *) (x))

static inline void __put_user(unsigned long x, void * y, int size)
{
        switch (size) {
	case 1:
                        __asm__ ("movb %b1,%%fs:%0"
				 :"=m" (*__sd(y))
				 :"iq" ((unsigned char) x), "m" (*__sd(y)));
                        break;
			case 2:
                        __asm__ ("movw %w1,%%fs:%0"
				 :"=m" (*__sd(y))
				 :"ir" ((unsigned short) x), "m" (*__sd(y)));
                        break;
			case 4:
                        __asm__ ("movl %1,%%fs:%0"
				 :"=m" (*__sd(y))
				 :"ir" (x), "m" (*__sd(y)));
                        break;
			default:
                        bad_user_access_length();
			}
	}
static inline unsigned long __get_user(const void * y, int size)
{
        unsigned long result;

        switch (size) {
	case 1:
                        __asm__ ("movb %%fs:%1,%b0"
				 :"=q" (result)
				 :"m" (*__const_sd(y)));
                        return (unsigned char) result;
			case 2:
                        __asm__ ("movw %%fs:%1,%w0"
				 :"=r" (result)
				 :"m" (*__const_sd(y)));
                        return (unsigned short) result;
			case 4:
                        __asm__ ("movl %%fs:%1,%0"
				 :"=r" (result)
				 :"m" (*__const_sd(y)));
                        return result;
			default:
                        return bad_user_access_length();
			}
	}


/*******************************************************************/
#endif

/* the "write" method had a "const" specifier added  in 1.3.15 */
#if LINUX_VERSION_CODE > 0x01030E
#define CONST_BUFFER const
#else
#define CONST_BUFFER /* nothing */
#endif 

